<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://themepure.net
 * @since      1.1.1
 *
 * @package    Tp_Wvs
 * @subpackage Tp_Wvs/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.1.1
 * @package    Tp_Wvs
 * @subpackage Tp_Wvs/includes
 * @author     ThemePure <themepure@gmail.com>
 */
require_once(plugin_dir_path(__DIR__) .'includes/class-pure-wc-swatches-helper.php');

class Tp_Wvs {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.1.1
	 * @access   protected
	 * @var      Tp_Wvs_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.1.1
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.1.1
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.1.1
	 */
	public function __construct() {
		if ( defined( 'TP_WVS_VERSION' ) ) {
			$this->version = TP_WVS_VERSION;
		} else {
			$this->version = '1.1.1';
		}
		$this->plugin_name = 'pure-wc-variation-swatches';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Tp_Wvs_Loader. Orchestrates the hooks of the plugin.
	 * - Tp_Wvs_i18n. Defines internationalization functionality.
	 * - Tp_Wvs_Admin. Defines all hooks for the admin area.
	 * - Tp_Wvs_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.1.1
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pure-wc-swatches-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pure-wc-swatches-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-pure-wc-swatches-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-pure-wc-swatches-public.php';

		$this->loader = new Tp_Wvs_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Tp_Wvs_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.1.1
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Tp_Wvs_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.1.1
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Tp_Wvs_Admin( $this->get_plugin_name(), $this->get_version() );
		$taxonomy = isset( $_REQUEST['taxonomy'] ) ? sanitize_title( $_REQUEST['taxonomy'] ) : '';

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
		$this->loader->add_action( 'admin_menu', $plugin_admin, 'admin_settings_menu' );
		$this->loader->add_action( $taxonomy . '_add_form_fields', $plugin_admin, 'add_form_fields' );
		$this->loader->add_action( $taxonomy . '_edit_form_fields', $plugin_admin, 'edit_form_fields', 10 );
		// Adding swatch types.
		$this->loader->add_filter( 'product_attributes_type_selector', $plugin_admin, 'add_swatch_types', 10, 1 );
		$this->loader->add_action( 'created_' .$taxonomy, $plugin_admin, 'save_term_fields');
		$this->loader->add_action( 'edited_' .$taxonomy, $plugin_admin, 'save_term_fields');
		$this->loader->add_filter( 'manage_edit-' . $taxonomy . '_columns', $plugin_admin, 'add_attribute_column');
		$this->loader->add_filter( 'manage_' . $taxonomy . '_custom_column', $plugin_admin, 'add_preview_markup', 10, 3 );
		$this->loader->add_action( 'wp_ajax_tpwvs_update_settings', $plugin_admin, 'tpwvs_update_settings' );
		$this->loader->add_action( 'wp_ajax_nopriv_tpwvs_update_settings', $plugin_admin, 'tpwvs_update_settings' );

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.1.1
	 * @access   private
	 */
	private function define_public_hooks() {
		$plugin_public = new Tp_Wvs_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );
		$this->loader->add_filter( 'woocommerce_dropdown_variation_attribute_options_html', $plugin_public, 'variation_html',20, 2 );
		$this->loader->add_filter( 'woocommerce_variation_is_active', $plugin_public, 'variation_check', 10, 2 );

		$get_options = TP_Wvs_Helper::get_option('tpwvs_shop');
		if($get_options['enable_swatches'] == true){
			$position = $get_options['swatch_position'];

			if ( ! empty( $position ) ) {
				switch ( $position ) {
					case 'after_thumbnail':
						$this->loader->add_action( 'woocommerce_shop_loop_item_title', $plugin_public, 'variation_attribute_html_shop_page',  9 );
						break;
					case 'after_title':
						$this->loader->add_action( 'woocommerce_shop_loop_item_title', $plugin_public, 'variation_attribute_html_shop_page',  11 );
						break;
					case 'after_rating':
						$this->loader->add_action( 'woocommerce_after_shop_loop_item_title', $plugin_public, 'variation_attribute_html_shop_page',  6 );
						break;
					case 'after_price':
						$this->loader->add_action( 'woocommerce_after_shop_loop_item_title', $plugin_public, 'variation_attribute_html_shop_page',  11 );
						break;
					case 'after_add_to_cart':
						$this->loader->add_action( 'woocommerce_after_shop_loop_item', $plugin_public, 'variation_attribute_html_shop_page',  11 );
						break;
					default:
						$this->loader->add_action( 'woocommerce_after_shop_loop_item_title', $plugin_public, 'variation_attribute_html_shop_page',  10 );
				}
			}
			
		}

		$this->loader->add_filter( 'woocommerce_loop_add_to_cart_args', $plugin_public, 'shop_page_add_to_cart_args', 10, 2 );
		$this->loader->add_action( 'wp_ajax_tpwvs_ajax_add_to_cart', $plugin_public, 'tpwvs_ajax_add_to_cart' );
		$this->loader->add_action( 'wp_ajax_nopriv_tpwvs_ajax_add_to_cart', $plugin_public, 'tpwvs_ajax_add_to_cart' );
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.1.1
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.1.1
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.1.1
	 * @return    Tp_Wvs_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.1.1
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
